@@ -1,7 +1,7 @@
 #! /usr/bin/perl
 
 # ex:ts=8 sw=4:
-# $OpenBSD: Quirks.pm,v 1.1065.2.3 2021/02/22 18:58:44 bket Exp $
+# $OpenBSD: Quirks.pm,v 1.1257 2021/04/07 11:43:12 sthen Exp $
 #
 # Copyright (c) 2009 Marc Espie <espie@openbsd.org>
 #
@@ -71,6 +71,8 @@ my $base_exceptions = {
 	'p5-Test-Tester' => "$p5/Test/Tester.pm",
 # 6.5
 	'libelf' => '/usr/lib/libelf.so.*',
+# 6.9
+	'acme-client' => '/usr/sbin/acme-client',
 };
 
 my $stem_extensions = {
@@ -342,7 +344,6 @@ my $stem_extensions = {
 # 6.7
 	'py-jaraco-functoolsjaraco.functools' => 'py-jaraco-functools',
 	'py3-jaraco-functoolsjaraco.functools' => 'py3-jaraco-functools',
-	'py3-qscintilla-qt5' => 'py3-qscintilla',
 	'py-netmiko' => 'py3-netmiko',
 	'py-napalm' => 'py3-napalm',
 	'py-IOSXR' => 'py3-IOSXR',
@@ -435,7 +436,6 @@ my $stem_extensions = {
 	'kdegraphics-thumbnailers' => 'kdegraphics-thumbnailers-kf5',
 	'svgpart' => 'svgpart-kf5',
 	'kdenetwork-filesharing' => 'kdenetwork-filesharing-kf5',
-	'spidermonkey60' => 'spidermonkey68',
 	'py-cloudpickle' => 'py3-cloudpickle',
 	'py-podcastparser' => 'py3-podcastparser',
 	'py-configargparse' => 'py3-configargparse',
@@ -479,6 +479,151 @@ my $stem_extensions = {
 	'py-urwid' => 'py3-urwid',
 	'py-ply' => 'py3-ply',
 	'py-dtfabric' => 'py3-dtfabric',
+# 6.9
+	'py-qscintilla' => 'py3-qscintilla',
+	'py-gobject3' => 'py3-gobject3',
+	'py-nltk' => 'py3-nltk',
+	'py-numpydoc' => 'py3-numpydoc',
+	'py-passlib' => 'py3-passlib',
+	'py-matplotlib' => 'py3-matplotlib',
+	'py-patsy' => 'py3-patsy',
+	'py-scs' => 'py3-scs',
+	'py-augeas' => 'py3-augeas',
+	'py-construct' => 'py3-construct',
+	'py-sane' => 'py3-sane',
+	'py-zmq' => 'py3-zmq',
+	'py-kiwisolver' => 'py3-kiwisolver',
+	'py-scipy' => 'py3-scipy',
+	'spidermonkey' => 'spidermonkey78',
+	'spidermonkey68' => 'spidermonkey78',
+	'py-relatorio' => 'py3-relatorio',
+	'py-croniter' => 'py3-croniter',
+	'biopython' => 'py3-biopython',
+	'py-rrd' => 'py-rrdtool',
+	'py-whisper' => 'py3-whisper',
+	'py-carbon' => 'py3-carbon',
+	'py-elftools' => 'py3-elftools',
+	'py-lockfile' => 'py3-lockfile',
+	'py-toml' => 'py3-toml',
+	'py-textfsm' => 'py3-textfsm',
+	'py-unicodecsv' => 'py3-unicodecsv',
+	'py-enchant' => 'py3-enchant',
+	'py-sqlparse' => 'py3-sqlparse',
+	'py-sqlalchemy-migrate' => 'py3-sqlalchemy-migrate',
+	'py-pygresql' => 'py3-pygresql',
+	'py-tagpy' => 'py3-tagpy',
+	'py-mpd2' => 'py3-mpd2',
+	'py-autobahn' => 'py3-autobahn',
+	'py-txaio' => 'py3-txaio',
+	'py-qrcode' => 'py3-qrcode',
+	'py-acoustid' => 'py3-acoustid',
+	'openjpeg' => 'openjp2',
+	'py-mysqlclient' => 'py3-mysqlclient',
+	'py-pyx' => 'py3-pyx',
+	'fvwm2+fvicons' => 'fvwm2',
+	'fvicons' => 'fvwm2',
+	'py-frozen-flask' => 'py3-frozen-flask',
+	'glibmm26' => 'glibmm268',
+	'docker' => 'docker-cli',
+	'py-lazy-object-proxy' => 'py3-lazy-project-proxy',
+	'py3-scapy' => 'scapy',
+	'py-sqlobject' => 'py3-sqlobject',
+	'pyglet' => 'py-pyglet',
+	'py-tzlocal' => 'py3-tzlocal',
+	'py-sphinx-notfound-page' => 'py3-sphinx-notfound-page',
+	'py-mulpyplexer' => 'py3-mulpyplexer',
+	'py-commonmark' => 'py3-commonmark',
+	'py-recommonmark' => 'py3-recommonmark',
+	'py-sphinx_guzzle_theme' => 'py3-sphinx_guzzle_theme',
+	'py-tornado' => 'py3-tornado',
+	'py-sphinx-intl' => 'py3-sphinx-intl',
+	'stegcracker' => 'stegseek',
+	'py-ldap3' => 'py3-ldap3',
+	'py-qtpy' => 'py3-qtpy',
+	'py-sip' => 'py3-sip',
+	'py-sip-qt5' => 'py3-pyqt5_sip',
+	'py3-sip-qt5' => 'py3-pyqt5_sip',
+	'py-qt5' => 'py3-qt5',
+	'py-ndg_httpsclient' => 'py3-ndg_httpsclient',
+	'py-trustme' => 'py3-trustme',
+	'py-mechanize' => 'py3-mechanize',
+	'py-ripe.atlas.cousteau' => 'py3-ripe.atlas.cousteau',
+	'py-ripe.atlas.sagan' => 'py3-ripe.atlas.sagan',
+	'py-qtawesome' => 'py3-qtawesome',
+	'py-twisted' => 'py3-twisted',
+	'py-hamcrest' => 'py3-hamcrest',
+	'py-automat' => 'py3-automat',
+	'py-constantly' => 'py3-constantly',
+	'py-incremental' => 'py3-incremental',
+	'py-zopeevent' => 'py3-zopeevent',
+	'py-zopeinterface' => 'py3-zopeinterface',
+	'py-zopetesting' => 'py3-zopetesting',
+	'py-zopecomponent' => 'py3-zopecomponent',
+	'py-service_identity' => 'py3-service_identity',
+	'py-hyperlink' => 'py3-hyperlink',
+	'py-asn1-modules' => 'py3-asn1-modules',
+	'py-ldap' => 'py3-ldap',
+	'py-ldap-examples' => 'py3-ldap-examples',
+	'py-pexpect' => 'py3-pexpect',
+	'py-gevent' => 'py3-gevent',
+	'py-statgrab' => 'py3-statgrab',
+	'py-parallel-ssh' => 'py3-parallel-ssh',
+	'py-geventhttpclient' => 'py3-geventhttpclient',
+	'py-gnupg' => 'py3-gnupg',
+	'py-babel' => 'py3-babel',
+	'py-imagesize' => 'py3-imagesize',
+	'py-alabaster' => 'py3-alabaster',
+	'py-docutils' => 'py3-docutils',
+	'py-pygments' => 'py3-pygments',
+	'py-snowballstemmer' => 'py3-snowballstemmer',
+	'py-stemmer' => 'py3-stemmer',
+	'py-sphinx' => 'py3-sphinx',
+	'py-sphinx_rtd_theme' => 'py3-sphinx_rtd_theme',
+	'py-codestyle' => 'py3-codestyle',
+	'pyflakes3' => 'pyflakes',
+	'py-dot' => 'py3-dot',
+	'py-last' => 'py3-last',
+	'py-flask-principal' => 'py3-flask-principal',
+	'py-flask-sqlalchemy' => 'py3-flask-sqlalchemy',
+	'py-flask-login' => 'py3-flask-login',
+	'py-flask-wtf' => 'py3-flask-wtf',
+	'py-pygal' => 'py3-pygal',
+	'py-wtforms' => 'py3-wtforms',
+	'py-apsw' => 'py3-apsw',
+	'cdiff' => 'ydiff',
+	'xmonad-lib' => 'xmonad',
+	'py-test-httpbin' => 'py3-test-httpbin',
+	'py-test-localserver' => 'py3-test-localserver',
+	'py-test-vcr' => 'py3-test-vcr',
+	'py-flask' => 'py3-flask',
+	'py-httpbin' => 'py3-httpbin',
+	'py-itsdangerous' => 'py3-itsdangerous',
+	'py-vcrpy' => 'py3-vcrpy',
+	'py-werkzeug' => 'py3-werkzeug',
+	'py-blinker' => 'py3-blinker',
+	'py-ifaddr' => 'py3-ifaddr',
+	'py-colorama' => 'py3-colorama',
+	'py-emoji' => 'py3-emoji',
+	'py-query' => 'py3-query',
+	'py-ofxparse' => 'py3-ofxparse',
+	'py-puppetboard' => 'py3-puppetboard',
+	'py-urlgrabber' => 'py3-urlgrabber',
+	'py-webob' => 'py3-webob',
+	'py-webtest' => 'py3-webtest',
+	'py-genshi' => 'py3-genshi',
+	'py-musicbrainzngs' => 'py3-musicbrainzngs',
+	'py-setproctitle' => 'py3-setproctitle',
+	'py-ptyprocess' => 'py3-ptyprocess',
+	'py-jwt' => 'py3-jwt',
+	'py-pg_activity' => 'pg_activity', # intentionally removing py- prefix
+	'py-axolotl' => 'py3-axolotl',
+	'py-protobuf' => 'py3-protobuf',
+	'py-http_ece' => 'py3-http_ece',
+	'yubiserve' => 'yubikeyedup',
+	'py-openssl' => 'py3-openssl',
+	'py-junos-eznc' => 'py3-junos-eznc',
+	'py-ncclient' => 'py3-ncclient',
+	'py-beautifulsoup4' => 'py3-beautifulsoup4',
 };
 
 my $obsolete_reason = {
@@ -939,7 +1084,7 @@ my $obsolete_reason = {
 	'xmms-fmradio' => 5,
 	'xmms-kj' => 5,
 	'xmms-mad' => 5,
-	'xmms-mikmod', => 5,
+	'xmms-mikmod' => 5,
 	'xmms-mp3' => 5,
 	'xmms-shn' => 5,
 	'xmms-sid' => 5,
@@ -1850,6 +1995,328 @@ my $obsolete_reason = {
 	'lazarus' => 0,
 	'fpc' => 5,
 	'exaile' => 13,
+	'instead-launcher' => 3,
+# 6.9
+	'py-notify' => 3,
+	'glimpse' => 3,
+	'py-metlog' => 3,
+	'pdfmod' => 3,
+	'hyena' => 5,
+	'mono-taglib' => 5,
+	'spatialite_gis' => 3,
+	'librewms' => 3,
+	'libgaiagraphics' => 3,
+	'librasterlite' => 3,
+	'dbus-sharp' => 5,
+	'dbus-sharp-glib' => 5,
+	'mono-gdata' => 5,
+	'mono-notify' => 5,
+	'ndesk-dbus' => 5,
+	'ndesk-dbus-glib' => 5,
+	'mono-xsp' => 5,
+	'py-Numeric' => 3,
+	'k3dsurf' => 3,
+	'spacehulk' => 3,
+	'taxipilot' => 3,
+	'pympd' => 3,
+	'py-vte' => 5,
+	'vte' => 5,
+	'keybinder' => 6,
+	'vinagre' => 3,
+	'gnome-getting-started-docs' => 5,
+	'py-poppler' => 3,
+	'pdfshuffler' => 3,
+	'colord' => 27,
+	'colord-gtk' => 27,
+	'gnome-color-manager' => 5,
+	'zeya' => 3,
+	'py-trollius' => 3,
+	'kde-i18n-af' => 3,
+	'kde-i18n-ar' => 3,
+	'kde-i18n-az' => 3,
+	'kde-i18n-be' => 3,
+	'kde-i18n-bg' => 3,
+	'kde-i18n-bn' => 3,
+	'kde-i18n-br' => 3,
+	'kde-i18n-bs' => 3,
+	'kde-i18n-ca' => 3,
+	'kde-i18n-cs' => 3,
+	'kde-i18n-csb' => 3,
+	'kde-i18n-cy' => 3,
+	'kde-i18n-da' => 3,
+	'kde-i18n-de' => 3,
+	'kde-i18n-el' => 3,
+	'kde-i18n-en_GB' => 3,
+	'kde-i18n-eo' => 3,
+	'kde-i18n-es' => 3,
+	'kde-i18n-et' => 3,
+	'kde-i18n-eu' => 3,
+	'kde-i18n-fa' => 3,
+	'kde-i18n-fi' => 3,
+	'kde-i18n-fr' => 3,
+	'kde-i18n-fy' => 3,
+	'kde-i18n-ga' => 3,
+	'kde-i18n-gl' => 3,
+	'kde-i18n-he' => 3,
+	'kde-i18n-hi' => 3,
+	'kde-i18n-hr' => 3,
+	'kde-i18n-hu' => 3,
+	'kde-i18n-is' => 3,
+	'kde-i18n-it' => 3,
+	'kde-i18n-ja' => 3,
+	'kde-i18n-kk' => 3,
+	'kde-i18n-km' => 3,
+	'kde-i18n-ko' => 3,
+	'kde-i18n-lt' => 3,
+	'kde-i18n-lv' => 3,
+	'kde-i18n-mk' => 3,
+	'kde-i18n-mn' => 3,
+	'kde-i18n-ms' => 3,
+	'kde-i18n-nb' => 3,
+	'kde-i18n-nds' => 3,
+	'kde-i18n-nl' => 3,
+	'kde-i18n-nn' => 3,
+	'kde-i18n-pa' => 3,
+	'kde-i18n-pl' => 3,
+	'kde-i18n-pt' => 3,
+	'kde-i18n-pt_BR' => 3,
+	'kde-i18n-ro' => 3,
+	'kde-i18n-ru' => 3,
+	'kde-i18n-rw' => 3,
+	'kde-i18n-se' => 3,
+	'kde-i18n-sk' => 3,
+	'kde-i18n-sl' => 3,
+	'kde-i18n-sr' => 3,
+	'kde-i18n-sr@Latn' => 3,
+	'kde-i18n-ss' => 3,
+	'kde-i18n-sv' => 3,
+	'kde-i18n-ta' => 3,
+	'kde-i18n-te' => 3,
+	'kde-i18n-tg' => 3,
+	'kde-i18n-th' => 3,
+	'kde-i18n-tr' => 3,
+	'kde-i18n-uk' => 3,
+	'kde-i18n-uz' => 3,
+	'kde-i18n-uz@cyrillic' => 3,
+	'kde-i18n-vi' => 3,
+	'kde-i18n-wa' => 3,
+	'kde-i18n-zh_CN' => 3,
+	'kde-i18n-zh_TW' => 3,
+	'kde-shared-data' => 3,
+	'kde3-langlist' => 3,
+	'kde3-locale' => 3,
+	'kde3-locale-en_US' => 3,
+	'kdeaccessibility' => 3,
+	'kdebase' => 3,
+	'kdegames' => 3,
+	'kdelibs' => 3,
+	'kdesamba' => 3,
+	'qt3-examples' => 3,
+	'qt3-html' => 3,
+	'qt3-mt' => 3,
+	'qt3-mysql' => 3,
+	'qt3-postgresql' => 3,
+	'qt3-sqlite2' => 3,
+	'gtk3mm-documentation' => 6,
+	'libxml++3' => 5,
+	'orage' => 3,
+	'xfce4-statusnotifier' => 3,
+	'pylint3-gui' => 3,
+	'e_dbus' => 3,
+	'ecore' => 3,
+	'edje' => 3,
+	'eet' => 3,
+	'efreet' => 3,
+	'eina' => 3,
+	'eio' => 3,
+	'elementary' => 3,
+	'embryo' => 3,
+	'emotion' => 3,
+	'enlightenment' => 3,
+	'ethumb' => 3,
+	'evas' => 3,
+	'qtstyleplugins' => 3,
+	'p5-Regexp-Copy' => 0,
+	'mcollective-puppet-agent' => 28,
+	'kdepim-apps-libs' => 3,
+	'gns3' => 29,
+	'py-qt4' => 3,
+	'py-qt4-docs' => 3,
+	'hitch' => 29,
+	'xtraceroute' => 30,
+	'py-vorbis' => 31,
+	'py-ogg' => 31,
+	'sattrack' => 32,
+	'sattrack' => 33,
+	'packit' => 34,
+	'nemesis' => 34,
+	'lxnb' => 6,
+	'siphon' => 35,
+	'avinfo' => 6,
+	'xfed' => 0,
+	'www6to4' => 36,
+	'gimmix' => 37,
+	'xhippo' => 3,
+	'hnb' => 0,
+	'icinga' => 38,
+	'icinga-web' => 38,
+	'icinga-cgi' => 38,
+	'icinga-idoutils' => 38,
+	'swftools' => 39,
+	'swfmill' => 39,
+	'ming' => 39,
+	'quvi' => 39,
+	'libquvi' => 39,
+	'libquvi-scripts' => 39,
+	'cclive' => 6,
+	'viz' => 4,
+	'driftnet' => 40,
+	'enjoympeg' => 6,
+	'trac-ldapplugin' => 6,
+	'xboing' => 0,
+	'beaver' => 3,
+	'tracnav' => 3,
+	'flasm' => 39,
+	'mpd-add-similar' => 31,
+	'py-mpd' => 31,
+	'abs' => 3,
+	'arena' => 3,
+	'teagtk' => 3,
+	'hs-cryptohash-sha256' => 5,
+	'hs-ed25519' => 5,
+	'hs-hackage-security' => 5,
+	'tempwatch' => 5,
+	'avenger' => 29,
+	'mk' => 41,
+	'sabredav' => 1,
+	'mollify' => 3,
+	'seyon' => 3,
+	'mingw' => 6,
+	'bsd-airtools' => 3,
+	'gtkpod' => 3,
+	'golem' => 0,
+	'py-beautifulsoup' => 14,
+	'wmmultipop3' => 3,
+	'wmpop3' => 3,
+	'angst' => 3,
+	'audiopreview' => 3,
+	'pcsxr' => 3,
+	'libbio' => 41,
+	'libfmt' => 41,
+	'libregexp9' => 41,
+	'libutf' => 41,
+	'hs-FindBin' => 5,
+	'hs-HTTP' => 5,
+	'hs-HUnit' => 5,
+	'hs-QuickCheck' => 5,
+	'hs-SHA' => 5,
+	'hs-X11' => 5,
+	'hs-X11-xft' => 5,
+	'hs-ansi-terminal' => 5,
+	'hs-ansi-wl-pprint' => 5,
+	'hs-async' => 5,
+	'hs-attoparsec' => 5,
+	'hs-base-compat' => 5,
+	'hs-base16-bytestring' => 5,
+	'hs-base64-bytestring' => 5,
+	'hs-byteable' => 5,
+	'hs-bytestring-show' => 5,
+	'hs-cereal' => 5,
+	'hs-cmdargs' => 5,
+	'hs-colour' => 5,
+	'hs-conduit' => 5,
+	'hs-constraints' => 5,
+	'hs-convertible' => 5,
+	'hs-crypto-api' => 5,
+	'hs-cryptohash' => 5,
+	'hs-data-default' => 5,
+	'hs-data-ordlist' => 5,
+	'hs-dbus' => 5,
+	'hs-digest' => 5,
+	'hs-dlist' => 5,
+	'hs-echo' => 5,
+	'hs-edit-distance' => 5,
+	'hs-enclosed-exceptions' => 5,
+	'hs-entropy' => 5,
+	'hs-exceptions' => 5,
+	'hs-extensible-exceptions' => 5,
+	'hs-fgl' => 5,
+	'hs-graphviz' => 5,
+	'hs-hashable' => 5,
+	'hs-hinotify' => 5,
+	'hs-hostname' => 5,
+	'hs-hslogger' => 5,
+	'hs-html' => 5,
+	'hs-json' => 5,
+	'hs-libmpd' => 5,
+	'hs-libxml-sax' => 5,
+	'hs-lifted-async' => 5,
+	'hs-lifted-base' => 5,
+	'hs-mmap' => 5,
+	'hs-mmorph' => 5,
+	'hs-monad-control' => 5,
+	'hs-mono-traversable' => 5,
+	'hs-nats' => 5,
+	'hs-network' => 5,
+	'hs-network-uri' => 5,
+	'hs-old-locale' => 5,
+	'hs-old-time' => 5,
+	'hs-polyparse' => 5,
+	'hs-primitive' => 5,
+	'hs-random' => 5,
+	'hs-regex-applicative' => 5,
+	'hs-regex-base' => 5,
+	'hs-regex-compat' => 5,
+	'hs-regex-compat-tdfa' => 5,
+	'hs-regex-posix' => 5,
+	'hs-regex-tdfa' => 5,
+	'hs-resourcet' => 5,
+	'hs-sandi' => 5,
+	'hs-scientific' => 5,
+	'hs-semigroups' => 5,
+	'hs-shelly' => 5,
+	'hs-split' => 5,
+	'hs-syb' => 5,
+	'hs-system-fileio' => 5,
+	'hs-system-filepath' => 5,
+	'hs-tagged' => 5,
+	'hs-tar' => 5,
+	'hs-temporary' => 5,
+	'hs-test-framework' => 5,
+	'hs-test-framework-hunit' => 5,
+	'hs-test-framework-quickcheck2' => 5,
+	'hs-tf-random' => 5,
+	'hs-transformers-base' => 5,
+	'hs-transformers-compat' => 5,
+	'hs-unix-compat' => 5,
+	'hs-unliftio-core' => 5,
+	'hs-unordered-containers' => 5,
+	'hs-utf8-string' => 5,
+	'hs-vector' => 5,
+	'hs-vector-algorithms' => 5,
+	'hs-void' => 5,
+	'hs-wl-pprint-text' => 5,
+	'hs-xml' => 5,
+	'hs-xml-types' => 5,
+	'hs-xmonad-contrib' => 5,
+	'hs-zip-archive' => 5,
+	'hs-zlib' => 5,
+	'gstreamer' => 6,
+	'gstreamer-ffmpeg' => 6,
+	'gstreamer-plugins-bad' => 6,
+	'gstreamer-plugins-base' => 6,
+	'gstreamer-plugins-gl' => 6,
+	'gstreamer-plugins-good' => 6,
+	'gstreamer-plugins-ugly' => 6,
+	'lives' => 42,
+	'py-wsgiproxy' => 5,
+	'py-sqlite' => 5,
+	'upt' => 43,
+	'upt-cpan' => 43,
+	'upt-openbsd' => 43,
+	'upt-pypi' => 43,
+	'upt-rubygems' => 43,
+	'p5-VCP-autrijus' => 3,
 };
 
 # reasons for obsolete packages
@@ -1881,6 +2348,23 @@ my @msg = (
 	"no longer maintained upstream, suggest qbirthday", #24
 	"no longer maintained upstream, suggest qmapshack", #25
 	"outdated Qt4 application, suggest keepassxc", #26
+	"requires GUdev or GUsb", #27
+	"consider migrating MCollective agents and filters using tools like Bolt and PuppetDB's Puppet Query Language", #28
+	"unmaintained port that was blocking other changes in ports", #29
+	"crashes in many different ways at runtime, does not build with '-fno-common'", #30
+	"upstream is dead, python2-only, nothing uses it", #31
+	"no more public distfiles, license forbids distributing package due to local changes", #32
+	"no longer maintained upstream, website suggests https://cronometer.com until rewrite is done", #33
+	"upstream is dead, basic functionality does not work, use tcpdump(1) and scapy(1) instead", #34
+	"upstream is dead, no longer useful, does not support IPv6", #35
+	"ancient software, use pf.conf(5) 'af-to'", #36
+	"upstream is dead, default settings cause crash when connecting to mpd", #37
+	"icinga 1 is end-of-life, migrate to icinga 2 (needs config rewrite)", #38
+	"Flash/SWF is end-of-life", #39
+	"ancient software that often crashes and relies on single HTTP (no TLS) connections, use wireshark", #40
+	"upstream moved to unversioned tarballs, use the plan9port (same upstream) package instead", #41
+	"unmaintained port that was blocking other changes in ports, suggest kdenlive or shotcut", #42
+	"using portgen instead is recommended", #43
 );
 
 # ->is_base_system($handle, $state):
@@ -1894,15 +2378,6 @@ sub is_base_system
 	my $pkgname = $handle->pkgname;
 	my $stem = OpenBSD::PackageName::splitstem($pkgname);
 
-	if ($stem eq 'ghc') {
-		require OpenBSD::PkgSpec;
-		my $spec = OpenBSD::PkgSpec->new('ghc-<8.2.2p5');
-		if ($spec->match_ref([$pkgname])) {
-			require OpenBSD::Quirks::ghc;
-			OpenBSD::Quirks::ghc::unfuck($handle, $state);
-		}
-	}
-
 	my $test = $base_exceptions->{$stem};
 	if (defined $test) {
 		require File::Glob;
@@ -1932,7 +2407,7 @@ sub filter_obsolete
 	for my $pkgname (@in) {
 		my $stem = OpenBSD::PackageName::splitstem($pkgname);
 		my $reason = $obsolete_reason->{$stem};
-		$reason = 3 if (!defined $reason && $pkgname =~ m/^(ruby(19|2[0-4])-|ruby-[^0-9])/);
+		$reason = 3 if (!defined $reason && $pkgname =~ m/^(ruby(19|2[0-5])-|ruby-[^0-9])/);
 		$reason = 21 if (!defined $reason && $pkgname =~ m/^drupal/);
 		if (defined $reason) {
 			$state->say("Obsolete package: #1 (#2)", $pkgname, 
@@ -1971,6 +2446,7 @@ sub tweak_search
 # list of
 #   cat/path => badspec
 my $cve = {
+	'archivers/brotli' => 'brotli-<1.0.9',
 	'archivers/cabextract' => 'cabextract-<1.8',
 	'archivers/libmspack' => 'libmspack-<0.8alpha',
 	'archivers/p5-Archive-Zip' => 'p5-Archive-Zip-<1.64',
@@ -1980,10 +2456,10 @@ my $cve = {
 	'databases/postgresql,-main' => 'postgresql-client-<10.6',
 	'databases/postgresql,-server' => 'postgresql-server-<10.6',
 	'databases/sqlite3' => 'sqlite3-<3.25.3',
-	'devel/apache-ant' => 'apache-ant-<1.10.8',
-	'devel/git,-main' => 'git-<2.26.2',
-	'devel/git,-svn' => 'git-svn-<2.26.2',
-	'devel/git,-x11' => 'git-x11-<2.26.2',
+	'devel/apache-ant' => 'apache-ant-<1.10.9',
+	'devel/git,-main' => 'git-<2.30.2',
+	'devel/git,-svn' => 'git-svn-<2.30.2',
+	'devel/git,-x11' => 'git-x11-<2.30.2',
 	'devel/jenkins/devel' => 'jenkins-<2.154',
 	'devel/jenkins/stable' => 'jenkins-<2.138.4',
 	'devel/libgit2/libgit2' => 'libgit2-<0.27.7',
@@ -1999,6 +2475,8 @@ my $cve = {
 	'lang/php/7.3,-main' => 'php->7.3,<7.3.5',
 	'lang/python/2.7,-main' => 'python->2.7,<2.7.16',
 	'lang/python/3.7,-main' => 'python->3.7,<3.7.9',
+	'lang/python/3.8,-main' => 'python->3.8,<3.8.8',
+	'lang/python/3.9,-main' => 'python->3.9,<3.9.2',
 	'lang/ruby/2.3,-main' => 'ruby-<2.3.8',
 	'lang/ruby/2.4,-main' => 'ruby->2.4,<2.4.5p2',
 	'lang/ruby/2.5,-main' => 'ruby->2.5,<2.5.5',
@@ -2017,6 +2495,7 @@ my $cve = {
 	'net/icecast' => 'icecast-<2.4.4',
 	'net/irssi' => 'irssi-<1.2.1',
 	'net/isc-bind' => 'isc-bind-<9.16.3',
+	'net/libssh' => 'libssh-<0.9.5',
 	'net/libssh2' => 'libssh2-<1.8.2',
 	'net/lldpd' => 'lldpd-<0.7.18p0',
 	'net/mosquitto' => 'mosquitto-<1.5.6',
@@ -2037,6 +2516,7 @@ my $cve = {
 	'net/wireshark,-gtk' => 'wireshark-gtk-<2.6.3',
 	'net/wireshark,-main' => 'wireshark-<2.6.3',
 	'net/wireshark,-text' => 'tshark-<2.6.3',
+	'net/zeromq' => 'zeromq-<4.3.3',
 	'net/znc' => 'znc-<1.7.3',
 	'print/cups,-main' => 'cups-<1.7.4',
 	'security/clamav' => 'clamav-<0.100.2',
@@ -2049,9 +2529,10 @@ my $cve = {
 	'sysutils/ansible,-main' => 'ansible-<2.7.1',
 	'sysutils/mcollective' => 'mcollective-<2.5.3',
 	'sysutils/rclone' => 'rclone-<1.53.3',
-	'sysutils/salt' => 'salt-<2018.3.3p2',
+	'sysutils/salt' => 'salt-<3002',
 	'telephony/asterisk,-main' => 'asterisk-<13.23.1',
 	'telephony/coturn' => 'turnserver-<4.5.1.2',
+	'textproc/mdbook' => 'mdbook-<0.4.6',
 	'www/apache-httpd,-main' => 'apache-httpd-<2.4.35',
 	'www/bozohttpd' => 'bozohttpd-<20130711p0',
 	'www/chromium' => 'chromium-<69.0.3497.100',
@@ -2081,8 +2562,6 @@ my $cve = {
 	'www/webkitgtk4' => 'webkitgtk4-<2.20.5',
 	'x11/gnome/gdm' => 'gdm-<3.28.3',
 	'x11/rdesktop' => 'rdesktop-<1.8.4',
-	'net/zeromq' => 'zeromq-<4.3.3',
-	'net/libssh' => 'libssh-<0.9.5',
 };
 # please maintain sort order in above $cve list, future updates need to
 # replace existing entries
